import { useRegisterTeacherMutation } from '@/app/backend/endpoints/auth';
import { Button } from '@/components/ui/button';
import { Input } from '@/components/ui/input';
import { Label } from '@/components/ui/label';
import { useFormik, Form, FormikProvider } from 'formik';
import { toast } from 'sonner';
import * as Yup from 'yup';
import {
  useGetTeacherByIdQuery,
  useUpdateTeacherMutation,
} from '@/app/backend/endpoints/accounts';
import Fallback from '@/components/Fallback';
import useUser from '@/hooks/useUser';
import { useNavigate } from 'react-router-dom';
import useTitle from '@/hooks/useTitle';

function CreateTeacher({
  isEdit = false,
  teacherId,
}: {
  isEdit?: boolean;
  teacherId?: string;
}) {
  useTitle('Create Teacher');
  const [Register, { isLoading }] = useRegisterTeacherMutation();
  const [Update, { isLoading: isUpdating }] = useUpdateTeacherMutation();
  const { user } = useUser();
  const { data: userData, isLoading: isLoadingUser } = useGetTeacherByIdQuery(
    parseInt(teacherId?.toString() ?? ''),
    {
      skip: !isEdit,
    }
  );
  const teacher = userData?.data;
  const navigate = useNavigate();
  const formik = useFormik<RegisterTeacher>({
    initialValues: {
      firstName: teacher?.firstName ?? '',
      lastName: teacher?.lastName ?? '',
      email: teacher?.email ?? '',
      password: '',
      role: 'teacher',
      phone: teacher?.phone ?? undefined,
      school_id: (user as SchoolI).school_id?.toString(),
    },
    validationSchema: Yup.object().shape({
      email: Yup.string().email().required('Please provide an email'),
      password: isEdit
        ? Yup.string().optional()
        : Yup.string().required('Please provide a password'),
      firstName: Yup.string().required('Please provide a first name'),
      lastName: Yup.string().required('Please provide a last name'),
      role: Yup.string().required('Please provide a role'),
      phone: Yup.string().optional(),
    }),
    enableReinitialize: true,
    validateOnMount: false,
    validateOnBlur: false,
    validateOnChange: false,
    onSubmit: (body: RegisterTeacher) => {
      if (body.phone === '' || body.phone === undefined) {
        body.phone = null;
      }
      if (isEdit) {
        delete body.password;

        Update({ teacherId: parseInt(teacherId!), body: body as UserI })
          .unwrap()
          .then((response) => {
            toast.success(response.message);
            navigate('/dashboard/teachers');
          })
          .catch((error: any) => {
            toast.error(error?.data?.message ?? error?.error.toString());
          });
        return;
      } else {
        Register(body as UserI)
          .unwrap()
          .then(() => {
            toast.success(`teacher created successfully`);
            navigate('/dashboard/teachers');
          })
          .catch((error: any) => {
            toast.error(error?.data?.message ?? error?.error.toString());
          });
      }
    },
  });
  const { getFieldProps, handleSubmit, touched, errors } = formik;

  if (isEdit && isLoadingUser) return <Fallback />;
  return (
    <>
      {/* <PageTitle title="Create User" /> */}
      <div className="flex justify-center items-center">
        <div className="w-full px-6">
          <FormikProvider value={formik}>
            <Form
              onSubmit={(e) => {
                e.preventDefault();
                handleSubmit(e);
              }}
            >
              <div className="grid grid-cols-2 gap-y-4 gap-x-6 py-5">
                {inputs.map(
                  ({ name, type = 'text', label, required, ...props }, i) => {
                    if (isEdit && name === 'password') return null;
                    return (
                      <div className="grid gap-2" key={i}>
                        <div className="flex items-center space-x-1">
                          <Label htmlFor={name}>{label}</Label>
                          {required! && <span className="text-red-500">*</span>}
                        </div>

                        <Input
                          type={type}
                          {...props}
                          {...getFieldProps(name)}
                          error={
                            touched[name] && errors[name]
                              ? errors[name]
                              : undefined
                          }
                        />
                      </div>
                    );
                  }
                )}
              </div>
              {!isLoading && !isLoadingUser && !isUpdating ? (
                <Button type="submit" className="w-full rounded-sm">
                  {isEdit ? 'Update' : 'Create'}
                </Button>
              ) : (
                <Button
                  disabled
                  type="submit"
                  className="w-full text-white focus:ring-4 font-medium rounded-sm text-sm px-5 py-2.5 text-center inline-flex items-center"
                >
                  <svg
                    aria-hidden="true"
                    role="status"
                    className="inline w-4 h-4 me-3 text-white animate-spin"
                    viewBox="0 0 100 101"
                    fill="none"
                    xmlns="http://www.w3.org/2000/svg"
                  >
                    <path
                      d="M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z"
                      fill="#E5E7EB"
                    />
                    <path
                      d="M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z"
                      fill="currentColor"
                    />
                  </svg>
                  Loading...
                </Button>
              )}
            </Form>
          </FormikProvider>
        </div>
      </div>
    </>
  );
}

const inputs: InputRequiredFields<RegisterTeacher>[] = [
  {
    required: true,
    name: 'firstName',
    placeholder: 'First Name',
    type: 'text',
    id: 'firstName',
    label: 'First Name',
    autoFocus: true,
  },
  {
    required: true,
    name: 'lastName',
    placeholder: 'Last Name',
    type: 'text',
    id: 'lastName',
    label: 'Last Name',
  },
  {
    required: true,
    type: 'email',
    id: 'email',
    placeholder: 'm@example.com',
    name: 'email',
    label: 'Email',
  },
  {
    required: true,
    name: 'password',
    placeholder: 'password',
    type: 'password',
    id: 'password',
    label: 'Password',
  },
  {
    required: false,
    name: 'phone',
    placeholder: 'Phone',
    type: 'text',
    id: 'phone',
    label: 'Phone',
  },
];
export default CreateTeacher;
